@@ -51,7 +51,12 @@ enum ENameValueFlag {
   /**
    * Tooltip was used as a name.
    */
-  eNameFromTooltip
+  eNameFromTooltip,
+
+  /**
+   * Labelling relations were used for the name.
+   */
+  eNameFromRelations,
 };
 
 /**
@@ -404,6 +409,9 @@ class Accessible {
 
   virtual Maybe<int32_t> GetIntARIAAttr(nsAtom* aAttrName) const = 0;
 
+  virtual bool GetStringARIAAttr(nsAtom* aAttrName,
+                                 nsAString& aAttrValue) const = 0;
+
   /**
    * Get the relation of the given type.
    */
@@ -442,11 +450,6 @@ class Accessible {
    */
   virtual nsAtom* TagName() const = 0;
 
-  /**
-   * Return input `type` attribute
-   */
-  virtual already_AddRefed<nsAtom> InputType() const = 0;
-
   /**
    * Return a landmark role if applied.
    */
@@ -646,8 +649,6 @@ class Accessible {
 
   bool IsDateTimeField() const { return mType == eHTMLDateTimeFieldType; }
 
-  bool IsSearchbox() const;
-
   virtual bool HasNumericValue() const = 0;
 
   /**
@@ -732,6 +733,14 @@ class Accessible {
   static void TranslateString(const nsString& aKey, nsAString& aStringOut,
                               const nsTArray<nsString>& aParams = {});
 
+  /*
+   * Return calculated group level based on accessible hierarchy.
+   *
+   * @param aFast  [in] Don't climb up tree. Calculate level from aria and
+   *                    roles.
+   */
+  virtual int32_t GetLevel(bool aFast) const;
+
  protected:
   // Some abstracted group utility methods.
 
@@ -751,14 +760,6 @@ class Accessible {
    */
   virtual AccGroupInfo* GetOrCreateGroupInfo() = 0;
 
-  /*
-   * Return calculated group level based on accessible hierarchy.
-   *
-   * @param aFast  [in] Don't climb up tree. Calculate level from aria and
-   *                    roles.
-   */
-  virtual int32_t GetLevel(bool aFast) const;
-
   /**
    * Calculate position in group and group size ('posinset' and 'setsize') based
    * on accessible hierarchy.
