@@ -142,23 +142,14 @@ exports.menuitems = [
         const ui = ResponsiveUIManager.getResponsiveUIForTab(
           commands.descriptorFront.localTab
         );
-        await ui.responsiveFront.setElementPickerState(
-          true,
-          PICKER_TYPES.EYEDROPPER
-        );
+        await ui.setElementPickerState(true, PICKER_TYPES.EYEDROPPER);
 
         inspectorFront.once("color-picked", async () => {
-          await ui.responsiveFront.setElementPickerState(
-            false,
-            PICKER_TYPES.EYEDROPPER
-          );
+          await ui.setElementPickerState(false, PICKER_TYPES.EYEDROPPER);
         });
 
         inspectorFront.once("color-pick-canceled", async () => {
-          await ui.responsiveFront.setElementPickerState(
-            false,
-            PICKER_TYPES.EYEDROPPER
-          );
+          await ui.setElementPickerState(false, PICKER_TYPES.EYEDROPPER);
         });
       }
 
