@@ -5,11 +5,12 @@
 #include "CCGCScheduler.h"
 
 #include "js/GCAPI.h"
-#include "mozilla/StaticPrefs_javascript.h"
 #include "mozilla/CycleCollectedJSRuntime.h"
-#include "mozilla/ProfilerMarkers.h"
-#include "mozilla/dom/ScriptSettings.h"
 #include "mozilla/PerfStats.h"
+#include "mozilla/ProfilerMarkers.h"
+#include "mozilla/StaticPrefs_javascript.h"
+#include "mozilla/dom/ScriptSettings.h"
+#include "mozilla/glean/DomMetrics.h"
 #include "nsRefreshDriver.h"
 
 /* Globally initialized constants
@@ -227,7 +228,7 @@ void CCGCScheduler::NoteGCSliceEnd(TimeStamp aStart, TimeStamp aEnd) {
   TimeDuration idleDuration = sliceDuration - nonIdleDuration;
   uint32_t percent =
       uint32_t(idleDuration.ToSeconds() / sliceDuration.ToSeconds() * 100);
-  Telemetry::Accumulate(Telemetry::GC_SLICE_DURING_IDLE, percent);
+  glean::dom::gc_slice_during_idle.AccumulateSingleSample(percent);
 
   mTriggeredGCDeadline.reset();
 }
@@ -1024,8 +1025,8 @@ JS::SliceBudget CCGCScheduler::ComputeForgetSkippableBudget(
     double duration =
         (endPoint - mForgetSkippableFrequencyStartTime).ToSeconds() / 60;
     uint32_t frequencyPerMinute = uint32_t(mForgetSkippableCounter / duration);
-    Telemetry::Accumulate(Telemetry::FORGET_SKIPPABLE_FREQUENCY,
-                          frequencyPerMinute);
+    glean::dom::forget_skippable_frequency.AccumulateSingleSample(
+        frequencyPerMinute);
     mForgetSkippableCounter = 0;
     mForgetSkippableFrequencyStartTime = aStartTimeStamp;
   }
