@@ -5,6 +5,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsIGlobalObject.h"
+
 #include "mozilla/BasePrincipal.h"
 #include "mozilla/CycleCollectedJSContext.h"
 #include "mozilla/GlobalFreezeObserver.h"
@@ -21,8 +22,8 @@
 #include "mozilla/dom/ServiceWorkerRegistration.h"
 #include "mozilla/ipc/PBackgroundSharedTypes.h"
 #include "nsContentUtils.h"
-#include "nsThreadUtils.h"
 #include "nsGlobalWindowInner.h"
+#include "nsThreadUtils.h"
 
 // Max number of Report objects
 constexpr auto MAX_REPORT_RECORDS = 100;
@@ -50,12 +51,11 @@ using mozilla::dom::ServiceWorkerRegistration;
 using mozilla::dom::ServiceWorkerRegistrationDescriptor;
 using mozilla::dom::VoidFunction;
 
-nsIGlobalObject::nsIGlobalObject()
-    : mIsDying(false), mIsScriptForbidden(false), mIsInnerWindow(false) {}
+nsIGlobalObject::nsIGlobalObject() : mIsDying(false), mIsInnerWindow(false) {}
 
 bool nsIGlobalObject::IsScriptForbidden(JSObject* aCallback,
                                         bool aIsJSImplementedWebIDL) const {
-  if (mIsScriptForbidden || mIsDying) {
+  if (mIsDying) {
     return true;
   }
 
@@ -106,11 +106,7 @@ class UnlinkHostObjectURIsRunnable final : public mozilla::Runnable {
 
   NS_IMETHOD Run() override {
     MOZ_ASSERT(NS_IsMainThread());
-
-    for (uint32_t index = 0; index < mURIs.Length(); ++index) {
-      BlobURLProtocolHandler::RemoveDataEntry(mURIs[index]);
-    }
-
+    BlobURLProtocolHandler::RemoveDataEntries(mURIs);
     return NS_OK;
   }
 
@@ -126,10 +122,7 @@ void nsIGlobalObject::UnlinkObjectsInGlobal() {
   if (!mHostObjectURIs.IsEmpty()) {
     // BlobURLProtocolHandler is main-thread only.
     if (NS_IsMainThread()) {
-      for (uint32_t index = 0; index < mHostObjectURIs.Length(); ++index) {
-        BlobURLProtocolHandler::RemoveDataEntry(mHostObjectURIs[index]);
-      }
-
+      BlobURLProtocolHandler::RemoveDataEntries(mHostObjectURIs);
       mHostObjectURIs.Clear();
     } else {
       RefPtr<UnlinkHostObjectURIsRunnable> runnable =
