@@ -6,30 +6,30 @@
 #ifndef CLIENTWEBGLCONTEXT_H_
 #define CLIENTWEBGLCONTEXT_H_
 
-#include "GLConsts.h"
-#include "js/GCAPI.h"
-#include "mozilla/dom/ImageData.h"
-#include "mozilla/Range.h"
-#include "mozilla/RefCounted.h"
-#include "mozilla/dom/TypedArray.h"
-#include "nsICanvasRenderingContextInternal.h"
-#include "nsWrapperCache.h"
-#include "mozilla/dom/WebGLRenderingContextBinding.h"
-#include "mozilla/dom/WebGL2RenderingContextBinding.h"
-#include "mozilla/layers/LayersSurfaces.h"
-#include "mozilla/StaticPrefs_webgl.h"
-#include "WebGLStrongTypes.h"
-#include "WebGLTypes.h"
-
-#include "mozilla/Logging.h"
-#include "WebGLCommandQueue.h"
-
 #include <memory>
 #include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <vector>
 
+#include "GLConsts.h"
+#include "WebGLCommandQueue.h"
+#include "WebGLStrongTypes.h"
+#include "WebGLTypes.h"
+#include "js/GCAPI.h"
+#include "mozilla/Logging.h"
+#include "mozilla/Range.h"
+#include "mozilla/RefCounted.h"
+#include "mozilla/StaticPrefs_webgl.h"
+#include "mozilla/dom/BufferSourceBindingFwd.h"
+#include "mozilla/dom/ImageData.h"
+#include "mozilla/dom/TypedArray.h"
+#include "mozilla/dom/WebGL2RenderingContextBinding.h"
+#include "mozilla/dom/WebGLRenderingContextBinding.h"
+#include "mozilla/layers/LayersSurfaces.h"
+#include "nsICanvasRenderingContextInternal.h"
+#include "nsWrapperCache.h"
+
 namespace mozilla {
 
 class ClientWebGLExtensionBase;
@@ -1003,15 +1003,19 @@ class ClientWebGLContext final : public nsICanvasRenderingContextInternal,
 
   void ResetBitmap() override;
 
-  UniquePtr<uint8_t[]> GetImageBuffer(int32_t* out_format,
-                                      gfx::IntSize* out_imageSize) override;
+  UniquePtr<uint8_t[]> GetImageBuffer(
+      mozilla::CanvasUtils::ImageExtraction aExtractionBehavior,
+      int32_t* out_format, gfx::IntSize* out_imageSize) override;
   NS_IMETHOD GetInputStream(const char* mimeType,
                             const nsAString& encoderOptions,
+                            mozilla::CanvasUtils::ImageExtraction spoofing,
                             nsIInputStream** out_stream) override;
 
   already_AddRefed<mozilla::gfx::SourceSurface> GetSurfaceSnapshot(
       gfxAlphaType* out_alphaType) override;
 
+  mozilla::ipc::IProtocol* SupportsSnapshotExternalCanvas() const override;
+
   void SetOpaqueValueFromOpaqueAttr(bool) override {};
   bool GetIsOpaque() override { return !mInitialOptions->alpha; }
 
@@ -1420,6 +1424,8 @@ class ClientWebGLContext final : public nsICanvasRenderingContextInternal,
 
   void Flush(bool flushGl = true) const;
 
+  void SyncSnapshot() override { Flush(); }
+
   void Finish();
 
   void FrontFace(GLenum mode);
@@ -1518,6 +1524,8 @@ class ClientWebGLContext final : public nsICanvasRenderingContextInternal,
                      GLuint srcElemCountOverride = 0);
   void BufferSubData(GLenum target, WebGLsizeiptr dstByteOffset,
                      const dom::ArrayBuffer& src);
+  void BufferSubData(GLenum target, WebGLsizeiptr dstByteOffset,
+                     const dom::AllowSharedBufferSource& src);
 
   void GetBufferSubData(GLenum target, GLintptr srcByteOffset,
                         const dom::ArrayBufferView& dstData,
@@ -2224,7 +2232,8 @@ class ClientWebGLContext final : public nsICanvasRenderingContextInternal,
                   dom::CallerType aCallerType, ErrorResult& out_error) const;
 
  protected:
-  bool ReadPixels_SharedPrecheck(dom::CallerType aCallerType,
+  bool ReadPixels_SharedPrecheck(GLenum* inout_readType,
+                                 dom::CallerType aCallerType,
                                  ErrorResult& out_error) const;
 
   // ------------------------------ Vertex Array ------------------------------
