@@ -7,19 +7,21 @@
 
 #include "GLBlitHelper.h"
 #include "GLContext.h"
-#include "mozilla/dom/Element.h"
-#include "mozilla/dom/HTMLCanvasElement.h"
-#include "mozilla/gfx/Logging.h"
-#include "mozilla/layers/ImageDataSerializer.h"
-#include "mozilla/layers/TextureHost.h"
-#include "mozilla/layers/VideoBridgeParent.h"
-#include "mozilla/RefPtr.h"
-#include "nsLayoutUtils.h"
 #include "WebGLBuffer.h"
 #include "WebGLContext.h"
 #include "WebGLFormats.h"
 #include "WebGLTexelConversions.h"
 #include "WebGLTexture.h"
+#include "mozilla/RefPtr.h"
+#include "mozilla/dom/Element.h"
+#include "mozilla/dom/HTMLCanvasElement.h"
+#include "mozilla/gfx/CanvasManagerParent.h"
+#include "mozilla/gfx/Logging.h"
+#include "mozilla/layers/ImageDataSerializer.h"
+#include "mozilla/layers/SharedSurfacesParent.h"
+#include "mozilla/layers/TextureHost.h"
+#include "mozilla/layers/VideoBridgeParent.h"
+#include "nsLayoutUtils.h"
 
 namespace mozilla {
 
@@ -314,6 +316,20 @@ static bool SDIsNullRemoteDecoder(const layers::SurfaceDescriptor& sd) {
                  .type() == layers::RemoteDecoderVideoSubDescriptor::Tnull_t;
 }
 
+// Check if the surface descriptor describes an ExternalImage surface for which
+// we only have an opaque source/handle to derive the actual surface from.
+static bool SDIsExternalImage(const layers::SurfaceDescriptor& sd) {
+  return sd.type() ==
+             layers::SurfaceDescriptor::TSurfaceDescriptorExternalImage &&
+         sd.get_SurfaceDescriptorExternalImage().source() ==
+             wr::ExternalImageSource::SharedSurfaces;
+}
+
+static bool SDIsCanvasSurface(const layers::SurfaceDescriptor& sd) {
+  return sd.type() ==
+         layers::SurfaceDescriptor::TSurfaceDescriptorCanvasSurface;
+}
+
 // static
 std::unique_ptr<TexUnpackBlob> TexUnpackBlob::Create(
     const TexUnpackBlobDesc& desc) {
@@ -334,17 +350,7 @@ std::unique_ptr<TexUnpackBlob> TexUnpackBlob::Create(
         return nullptr;
     }
 
-    if (desc.sd) {
-      // Shmem buffers need to be treated as if they were a DataSourceSurface.
-      // Otherwise, TexUnpackImage will try to blit the surface descriptor as
-      // if it can be mapped as a framebuffer, whereas the Shmem is still CPU
-      // data.
-      if (SDIsRGBBuffer(*desc.sd) || SDIsNullRemoteDecoder(*desc.sd)) {
-        return new TexUnpackSurface(desc);
-      }
-      return new TexUnpackImage(desc);
-    }
-    if (desc.dataSurf) {
+    if (desc.sd || desc.sourceSurf) {
       return new TexUnpackSurface(desc);
     }
 
@@ -408,6 +414,12 @@ bool TexUnpackBlob::ConvertIfNeeded(
     dstOrigin = srcOrigin;
   }
 
+  // TODO (Bug 754256): Figure out the source colorSpace.
+  dom::PredefinedColorSpace srcColorSpace = dom::PredefinedColorSpace::Srgb;
+  dom::PredefinedColorSpace dstColorSpace =
+      webgl->mUnpackColorSpace ? *webgl->mUnpackColorSpace
+                               : dom::PredefinedColorSpace::Srgb;
+
   if (srcFormat != dstFormat) {
     webgl->GeneratePerfWarning(
         "Conversion requires pixel reformatting. (%u->%u)", uint32_t(srcFormat),
@@ -421,6 +433,10 @@ bool TexUnpackBlob::ConvertIfNeeded(
   } else if (srcStride != dstStride) {
     webgl->GeneratePerfWarning("Conversion requires change in stride. (%u->%u)",
                                uint32_t(srcStride), uint32_t(dstStride));
+  } else if (srcColorSpace != dstColorSpace) {
+    webgl->GeneratePerfWarning(
+        "Conversion requires colorSpace conversion. (%u->%u)",
+        uint32_t(srcColorSpace), uint32_t(dstColorSpace));
   } else {
     return true;
   }
@@ -446,7 +462,8 @@ bool TexUnpackBlob::ConvertIfNeeded(
   bool wasTrivial;
   if (!ConvertImage(rowLength, rowCount, srcBegin, srcStride, srcOrigin,
                     srcFormat, srcIsPremult, dstBegin, dstStride, dstOrigin,
-                    dstFormat, dstIsPremult, &wasTrivial)) {
+                    dstFormat, dstIsPremult, srcColorSpace, dstColorSpace,
+                    &wasTrivial)) {
     webgl->ErrorImplementationBug("ConvertImage failed.");
     return false;
   }
@@ -643,25 +660,25 @@ bool TexUnpackBytes::TexOrSubImage(bool isSubImage, bool needsRespec,
 
 ////////////////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
-// TexUnpackImage
+// TexUnpackSurface
 
-TexUnpackImage::~TexUnpackImage() = default;
+TexUnpackSurface::~TexUnpackSurface() = default;
 
-bool TexUnpackImage::Validate(const WebGLContext* const webgl,
-                              const webgl::PackingInfo& pi) {
+bool TexUnpackSurface::Validate(const WebGLContext* const webgl,
+                                const webgl::PackingInfo& pi) {
   if (!ValidatePIForDOM(webgl, pi)) return false;
 
   if (!mDesc.structuredSrcSize) {
-    gfxCriticalError() << "TexUnpackImage missing structuredSrcSize.";
+    gfxCriticalError() << "TexUnpackSurface missing structuredSrcSize.";
     return false;
   }
   const auto& elemSize = *mDesc.structuredSrcSize;
-  if (mDesc.dataSurf) {
-    const auto& surfSize = mDesc.dataSurf->GetSize();
+  if (mDesc.sourceSurf) {
+    const auto& surfSize = mDesc.sourceSurf->GetSize();
     const auto surfSize2 = ivec2::FromSize(surfSize)->StaticCast<uvec2>();
     if (uvec2{elemSize.x, elemSize.y} != surfSize2) {
       gfxCriticalError()
-          << "TexUnpackImage mismatched structuredSrcSize for dataSurf.";
+          << "TexUnpackSurface mismatched structuredSrcSize for sourceSurf.";
       return false;
     }
   }
@@ -670,149 +687,231 @@ bool TexUnpackImage::Validate(const WebGLContext* const webgl,
   return ValidateUnpackPixels(webgl, pi, fullRows, *this);
 }
 
-Maybe<std::string> BlitPreventReason(
-    const int32_t level, const ivec3& offset, const GLenum internalFormat,
+const char* BlitPreventReason(
+    GLenum target, int32_t level, const ivec3& offset, GLenum internalFormat,
     const webgl::PackingInfo& pi, const TexUnpackBlobDesc& desc,
-    const OptionalRenderableFormatBits optionalRenderableFormatBits) {
+    OptionalRenderableFormatBits optionalRenderableFormatBits,
+    bool sameColorSpace, bool allowConversion, bool allowSRGB, bool allow3D) {
   const auto& size = desc.size;
   const auto& unpacking = desc.unpacking;
 
-  const auto ret = [&]() -> const char* {
-    if (size.z != 1) {
-      return "depth is not 1";
+  if (size.z != 1) {
+    return "depth is not 1";
+  }
+  if (offset.z != 0) {
+    return "zOffset is not 0";
+  }
+
+  if (unpacking.skipPixels || unpacking.skipRows || unpacking.skipImages) {
+    return "non-zero UNPACK_SKIP_* not yet supported";
+  }
+
+  if (desc.srcAlphaType != gfxAlphaType::Opaque) {
+    const bool srcIsPremult = (desc.srcAlphaType == gfxAlphaType::Premult);
+    const auto& dstIsPremult = unpacking.premultiplyAlpha;
+    if (srcIsPremult != dstIsPremult && !allowConversion) {
+      return dstIsPremult ? "UNPACK_PREMULTIPLY_ALPHA_WEBGL is not true"
+                          : "UNPACK_PREMULTIPLY_ALPHA_WEBGL is not false";
     }
-    if (offset.x != 0 || offset.y != 0 || offset.z != 0) {
-      return "x/y/zOffset is not 0";
+  }
+
+  if (!sameColorSpace) {
+    return "not same colorSpace";
+  }
+
+  const auto formatReason = [&]() -> const char* {
+    if (pi.type != LOCAL_GL_UNSIGNED_BYTE) {
+      return "`unpackType` must be `UNSIGNED_BYTE`";
     }
 
-    if (unpacking.skipPixels || unpacking.skipRows || unpacking.skipImages) {
-      return "non-zero UNPACK_SKIP_* not yet supported";
+    // GL_FRAMEBUFFER_SRGB is not available in some GLES implementations,
+    // which is required for blitting to emulate Tex(Sub)Image, which does no
+    // encoding when uploading to an sRGB texture. In GLES (but not desktop
+    // GL) blitting to an sRGB framebuffer encodes by default unless
+    // explicitly disabled. To workaround the availability of this extension,
+    // just disallow it for now. Note, this is also an optional boolean as
+    // some internal data transfers intentionally rely on this conversion.
+    switch (internalFormat) {
+      case LOCAL_GL_SRGB:
+      case LOCAL_GL_SRGB8:
+      case LOCAL_GL_SRGB_ALPHA:
+      case LOCAL_GL_SRGB8_ALPHA8:
+        if (!allowSRGB) {
+          return "sRGB-encoded internal formats are not supported";
+        }
+        break;
     }
 
-    const auto premultReason = [&]() -> const char* {
-      if (desc.srcAlphaType == gfxAlphaType::Opaque) return nullptr;
+    switch (pi.format) {
+      case LOCAL_GL_RGBA:
+        return nullptr;  // All internalFormats for unpackFormat=RGBA are
+                         // renderable.
 
-      const bool srcIsPremult = (desc.srcAlphaType == gfxAlphaType::Premult);
-      const auto& dstIsPremult = unpacking.premultiplyAlpha;
-      if (srcIsPremult == dstIsPremult) return nullptr;
+      case LOCAL_GL_RGB:
+        break;
 
-      if (dstIsPremult) {
-        return "UNPACK_PREMULTIPLY_ALPHA_WEBGL is not true";
-      } else {
-        return "UNPACK_PREMULTIPLY_ALPHA_WEBGL is not false";
-      }
-    }();
-    if (premultReason) return premultReason;
+      default:
+        return "`unpackFormat` must be `RGBA` or maybe `RGB`";
+    }
 
-    const auto formatReason = [&]() -> const char* {
-      if (pi.type != LOCAL_GL_UNSIGNED_BYTE) {
-        return "`unpackType` must be `UNSIGNED_BYTE`";
-      }
+    // -
 
-      switch (pi.format) {
-        case LOCAL_GL_RGBA:
-          return nullptr;  // All internalFormats for unpackFormat=RGBA are
-                           // renderable.
+    struct {
+      OptionalRenderableFormatBits bits;
+      const char* errorMsg;
+    } required;
 
-        case LOCAL_GL_RGB:
-          break;
+    switch (internalFormat) {
+      case LOCAL_GL_RGB565:
+        return nullptr;
+      case LOCAL_GL_RGB:
+      case LOCAL_GL_RGB8:
+        required = {
+            OptionalRenderableFormatBits::RGB8,
+            "Unavailable, as blitting internalFormats RGB or RGB8 requires "
+            "that RGB8 must be a renderable format.",
+        };
+        break;
+      case LOCAL_GL_SRGB:
+      case LOCAL_GL_SRGB8:
+        required = {
+            OptionalRenderableFormatBits::SRGB8,
+            "Unavailable, as blitting internalFormats SRGB or SRGB8 requires "
+            "that SRGB8 must be a renderable format.",
+        };
+        break;
+      case 0:
+        // texSubImage, so internalFormat is unknown, and could be anything!
+        required = {
+            OptionalRenderableFormatBits::RGB8 |
+                OptionalRenderableFormatBits::SRGB8,
+            "Unavailable, as blitting texSubImage with unpackFormat=RGB "
+            "requires that RGB8 and SRGB8 must be renderable formats.",
+        };
+        break;
+      default:
+        gfxCriticalError()
+            << "Unexpected internalFormat for unpackFormat=RGB: 0x"
+            << gfx::hexa(internalFormat);
+        return "Unexpected internalFormat for unpackFormat=RGB";
+    }
 
-        default:
-          return "`unpackFormat` must be `RGBA` or maybe `RGB`";
-      }
+    const auto availableBits = optionalRenderableFormatBits;
+    if ((required.bits | availableBits) != availableBits) {
+      return required.errorMsg;
+    }
 
-      // -
-
-      struct {
-        OptionalRenderableFormatBits bits;
-        const char* errorMsg;
-      } required;
-
-      switch (internalFormat) {
-        case LOCAL_GL_RGB565:
-          return nullptr;
-        case LOCAL_GL_RGB:
-        case LOCAL_GL_RGB8:
-          required = {
-              OptionalRenderableFormatBits::RGB8,
-              "Unavailable, as blitting internalFormats RGB or RGB8 requires "
-              "that RGB8 must be a renderable format.",
-          };
-          break;
-        case LOCAL_GL_SRGB:
-        case LOCAL_GL_SRGB8:
-          required = {
-              OptionalRenderableFormatBits::SRGB8,
-              "Unavailable, as blitting internalFormats SRGB or SRGB8 requires "
-              "that SRGB8 must be a renderable format.",
-          };
-          break;
-        case 0:
-          // texSubImage, so internalFormat is unknown, and could be anything!
-          required = {
-              OptionalRenderableFormatBits::RGB8 |
-                  OptionalRenderableFormatBits::SRGB8,
-              "Unavailable, as blitting texSubImage with unpackFormat=RGB "
-              "requires that RGB8 and SRGB8 must be renderable formats.",
-          };
-          break;
-        default:
-          gfxCriticalError()
-              << "Unexpected internalFormat for unpackFormat=RGB: 0x"
-              << gfx::hexa(internalFormat);
-          return "Unexpected internalFormat for unpackFormat=RGB";
-      }
-
-      const auto availableBits = optionalRenderableFormatBits;
-      if ((required.bits | availableBits) != availableBits) {
-        return required.errorMsg;
-      }
-
-      // -
-
-      return nullptr;
-    }();
-    if (formatReason) return formatReason;
+    // -
 
     return nullptr;
   }();
-  if (ret) {
-    return Some(std::string(ret));
+  if (formatReason) return formatReason;
+
+  // If the texture has multiple faces or layers that are not allocated,
+  // then the framebuffer attachment required to blit the descriptor may
+  // be determined incomplete. Avoid this for now by restricting to 2D
+  // targets.
+  if (!allow3D) {
+    bool is2D = ImageToTexTarget(target) == target && !IsTexTarget3D(target);
+    if (!is2D) {
+      return "texture is not 2D";
+    }
   }
-  return {};
+
+  return nullptr;
 }
 
-bool TexUnpackImage::TexOrSubImage(bool isSubImage, bool needsRespec,
-                                   WebGLTexture* tex, GLint level,
-                                   const webgl::DriverUnpackInfo* dui,
-                                   GLint xOffset, GLint yOffset, GLint zOffset,
+bool TexUnpackSurface::AllowBlitSd(WebGLContext* const webgl,
+                                   const GLenum target, const int32_t level,
+                                   const ivec3& offset,
+                                   const GLenum internalFormat,
                                    const webgl::PackingInfo& pi,
-                                   GLenum* const out_error) const {
+                                   bool allowConversion, bool allowSRGB,
+                                   bool allow3D, bool warn) const {
+  dom::PredefinedColorSpace srcColorSpace = dom::PredefinedColorSpace::Srgb;
+  dom::PredefinedColorSpace dstColorSpace =
+      webgl->mUnpackColorSpace ? *webgl->mUnpackColorSpace
+                               : dom::PredefinedColorSpace::Srgb;
+  bool sameColorSpace = srcColorSpace == dstColorSpace;
+  if (const char* reason = BlitPreventReason(
+          target, level, offset, internalFormat, pi, mDesc,
+          webgl->mOptionalRenderableFormatBits, sameColorSpace, allowConversion,
+          allowSRGB, allow3D)) {
+    if (warn) {
+      webgl->GeneratePerfWarning(
+          "Failed to hit GPU-copy fast-path."
+          " (%s) Falling back to CPU upload.",
+          reason);
+    }
+    return false;
+  }
+  return true;
+}
+
+// The texture may be mipmap incomplete which will prevent the framebuffer
+// from being complete while drawing to it. To avoid this scenario, override
+// the texture base and max level temporarily to ignore incomplete mipmaps
+// while blitting to it. Depending on GL implementation (desktop vs ES), the
+// min filter may contribute to mipmap completeness.
+struct AutoRestoreMipmapState {
+  AutoRestoreMipmapState(gl::GLContext* gl, GLenum target, GLint level)
+      : mGL(gl), mTarget(target), mLevel(level) {
+    mGL->fGetTexParameteriv(mTarget, LOCAL_GL_TEXTURE_MIN_FILTER, &mMinFilter);
+    if (IsTexMipmapFilter(mMinFilter)) {
+      mGL->fTexParameteri(mTarget, LOCAL_GL_TEXTURE_MIN_FILTER,
+                          LOCAL_GL_NEAREST);
+    }
+    if (mGL->HasTexParamMipmapLevel()) {
+      mGL->fGetTexParameteriv(mTarget, LOCAL_GL_TEXTURE_BASE_LEVEL,
+                              &mLevelBase);
+      mGL->fGetTexParameteriv(mTarget, LOCAL_GL_TEXTURE_MAX_LEVEL, &mLevelMax);
+      if (mLevelBase != mLevel) {
+        mGL->fTexParameteri(mTarget, LOCAL_GL_TEXTURE_BASE_LEVEL, mLevel);
+      }
+      if (mLevelMax != mLevel) {
+        mGL->fTexParameteri(mTarget, LOCAL_GL_TEXTURE_MAX_LEVEL, mLevel);
+      }
+    }
+  }
+
+  ~AutoRestoreMipmapState() {
+    if (IsTexMipmapFilter(mMinFilter)) {
+      mGL->fTexParameteri(mTarget, LOCAL_GL_TEXTURE_MIN_FILTER, mMinFilter);
+    }
+    if (mGL->HasTexParamMipmapLevel()) {
+      if (mLevelBase != mLevel) {
+        mGL->fTexParameteri(mTarget, LOCAL_GL_TEXTURE_BASE_LEVEL, mLevelBase);
+      }
+      if (mLevelMax != mLevel) {
+        mGL->fTexParameteri(mTarget, LOCAL_GL_TEXTURE_MAX_LEVEL, mLevelMax);
+      }
+    }
+  }
+
+  gl::GLContext* mGL = nullptr;
+  GLenum mTarget = 0;
+  GLint mLevel = 0;
+  GLint mMinFilter = 0;
+  GLint mLevelBase = 0;
+  GLint mLevelMax = 0;
+};
+
+bool TexUnpackSurface::BlitSd(
+    const layers::SurfaceDescriptor& sd, bool isSubImage, bool needsRespec,
+    WebGLTexture* tex, GLint level, const webgl::DriverUnpackInfo* dui,
+    GLint xOffset, GLint yOffset, GLint zOffset, const webgl::PackingInfo& pi,
+    GLenum* const out_error, bool allowFallback) const {
   MOZ_ASSERT_IF(needsRespec, !isSubImage);
 
   const auto& webgl = tex->mContext;
   const auto& target = mDesc.imageTarget;
   const auto& size = mDesc.size;
-  const auto& sd = *(mDesc.sd);
   const auto& unpacking = mDesc.unpacking;
 
   const auto& gl = webgl->GL();
 
   // -
 
-  const auto reason =
-      BlitPreventReason(level, {xOffset, yOffset, zOffset}, dui->internalFormat,
-                        pi, mDesc, webgl->mOptionalRenderableFormatBits);
-  if (reason) {
-    webgl->GeneratePerfWarning(
-        "Failed to hit GPU-copy fast-path."
-        " (%s) Falling back to CPU upload.",
-        reason->c_str());
-    return false;
-  }
-
-  // -
-
   if (needsRespec) {
     *out_error =
         DoTexOrSubImage(isSubImage, gl, target, level, dui, xOffset, yOffset,
@@ -821,6 +920,9 @@ bool TexUnpackImage::TexOrSubImage(bool isSubImage, bool needsRespec,
   }
 
   {
+    gl::ScopedBindTexture scopedTex(gl, tex->mGLName, target);
+    AutoRestoreMipmapState restoreMipmapState(gl, target, level);
+
     gl::ScopedFramebuffer scopedFB(gl);
     gl::ScopedBindFramebuffer bindFB(gl, scopedFB.FB());
 
@@ -832,18 +934,46 @@ bool TexUnpackImage::TexOrSubImage(bool isSubImage, bool needsRespec,
                                 tex->mGLName, level);
 
       const auto err = errorScope.GetError();
-      MOZ_ALWAYS_TRUE(!err);
+      if (err) {
+        if (allowFallback) {
+          return false;
+        }
+        MOZ_DIAGNOSTIC_CRASH("BlitSd failed attaching texture to framebuffer");
+      }
     }
 
     const GLenum status = gl->fCheckFramebufferStatus(LOCAL_GL_FRAMEBUFFER);
-    MOZ_ALWAYS_TRUE(status == LOCAL_GL_FRAMEBUFFER_COMPLETE);
+    if (status != LOCAL_GL_FRAMEBUFFER_COMPLETE) {
+      if (allowFallback) {
+        return false;
+      }
+      MOZ_DIAGNOSTIC_CRASH("BlitSd framebuffer is not complete");
+    }
 
     const auto dstOrigin =
         (unpacking.flipY ? gl::OriginPos::TopLeft : gl::OriginPos::BottomLeft);
-    if (!gl->BlitHelper()->BlitSdToFramebuffer(sd, {size.x, size.y},
-                                               dstOrigin)) {
+    gfx::IntSize fbSize(size.x, size.y);
+    if (isSubImage) {
+      const auto& imageInfo = tex->ImageInfoAt(target, level);
+      fbSize = gfx::IntSize(imageInfo.mWidth, imageInfo.mHeight);
+    }
+    Maybe<gfxAlphaType> convertAlpha;
+    if (mDesc.srcAlphaType != gfxAlphaType::Opaque) {
+      const bool srcIsPremult = mDesc.srcAlphaType == gfxAlphaType::Premult;
+      const bool dstIsPremult = unpacking.premultiplyAlpha;
+      if (srcIsPremult != dstIsPremult) {
+        convertAlpha = Some(dstIsPremult ? gfxAlphaType::Premult
+                                         : gfxAlphaType::NonPremult);
+      }
+    }
+    if (!gl->BlitHelper()->BlitSdToFramebuffer(
+            sd, {xOffset, yOffset, size.x, size.y}, dstOrigin, fbSize,
+            convertAlpha)) {
       gfxCriticalNote << "BlitSdToFramebuffer failed for type "
                       << int(sd.type());
+      if (allowFallback) {
+        return false;
+      }
       // Maybe the resource isn't valid anymore?
       gl->fClearColor(0.2, 0.0, 0.2, 1.0);
       gl->fClear(LOCAL_GL_COLOR_BUFFER_BIT);
@@ -858,14 +988,6 @@ bool TexUnpackImage::TexOrSubImage(bool isSubImage, bool needsRespec,
   return true;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////
-// TexUnpackSurface
-
-TexUnpackSurface::~TexUnpackSurface() = default;
-
-//////////
-
 static bool GetFormatForSurf(const gfx::SourceSurface* surf,
                              WebGLTexelFormat* const out_texelFormat,
                              uint8_t* const out_bpp) {
@@ -901,7 +1023,7 @@ static bool GetFormatForSurf(const gfx::SourceSurface* surf,
       *out_bpp = 1;
       return true;
 
-    case gfx::SurfaceFormat::YUV:
+    case gfx::SurfaceFormat::YUV420:
       // Ugh...
       NS_ERROR("We don't handle uploads from YUV sources yet.");
       // When we want to, check out gfx/ycbcr/YCbCrUtils.h. (specifically
@@ -913,31 +1035,6 @@ static bool GetFormatForSurf(const gfx::SourceSurface* surf,
   }
 }
 
-//////////
-
-bool TexUnpackSurface::Validate(const WebGLContext* const webgl,
-                                const webgl::PackingInfo& pi) {
-  if (!ValidatePIForDOM(webgl, pi)) return false;
-
-  if (!mDesc.structuredSrcSize) {
-    gfxCriticalError() << "TexUnpackSurface missing structuredSrcSize.";
-    return false;
-  }
-  const auto& elemSize = *mDesc.structuredSrcSize;
-  if (mDesc.dataSurf) {
-    const auto& surfSize = mDesc.dataSurf->GetSize();
-    const auto surfSize2 = ivec2::FromSize(surfSize)->StaticCast<uvec2>();
-    if (uvec2{elemSize.x, elemSize.y} != surfSize2) {
-      gfxCriticalError()
-          << "TexUnpackSurface mismatched structuredSrcSize for dataSurf.";
-      return false;
-    }
-  }
-
-  const auto fullRows = elemSize.y;
-  return ValidateUnpackPixels(webgl, pi, fullRows, *this);
-}
-
 bool TexUnpackSurface::TexOrSubImage(bool isSubImage, bool needsRespec,
                                      WebGLTexture* tex, GLint level,
                                      const webgl::DriverUnpackInfo* dui,
@@ -949,9 +1046,39 @@ bool TexUnpackSurface::TexOrSubImage(bool isSubImage, bool needsRespec,
   const auto& size = mDesc.size;
   RefPtr<gfx::DataSourceSurface> surf;
   if (mDesc.sd) {
-    // If we get here, we assume the SD describes an RGBA Shmem.
+    // First check if the SD describes an RGBA Shmem.
     const auto& sd = *(mDesc.sd);
-    if (SDIsRGBBuffer(sd)) {
+    if (SDIsCanvasSurface(sd)) {
+      // The canvas surface resides on a 2D canvas within the same content
+      // process as the WebGL canvas. Query it for the surface.
+      const auto& sdc = sd.get_SurfaceDescriptorCanvasSurface();
+      uint32_t managerId = sdc.managerId();
+      mozilla::ipc::ActorId canvasId = sdc.canvasId();
+      uintptr_t surfaceId = sdc.surfaceId();
+      Maybe<layers::SurfaceDescriptor> exportSd;
+      // If the texture has multiple faces or layers that are not allocated,
+      // then the framebuffer attachment required to blit the descriptor may
+      // be determined incomplete. Avoid this for now by restricting to 2D
+      // targets.
+      bool allowBlit = AllowBlitSd(
+          webgl, mDesc.imageTarget, level, {xOffset, yOffset, zOffset},
+          dui->internalFormat, dstPI, true, false, false, false);
+      if (RefPtr<gfx::SourceSurface> data =
+              gfx::CanvasManagerParent::GetCanvasSurface(
+                  webgl->GetContentId(), managerId, canvasId, surfaceId,
+                  allowBlit ? &exportSd : nullptr)) {
+        if (exportSd && !SDIsRGBBuffer(*exportSd) &&
+            BlitSd(*exportSd, isSubImage, needsRespec, tex, level, dui, xOffset,
+                   yOffset, zOffset, dstPI, out_error, true)) {
+          return true;
+        }
+        surf = data->GetDataSurface();
+      }
+      if (!surf) {
+        gfxCriticalError() << "TexUnpackSurface failed to get CanvasSurface";
+        return false;
+      }
+    } else if (SDIsRGBBuffer(sd)) {
       const auto& sdb = sd.get_SurfaceDescriptorBuffer();
       const auto& rgb = sdb.desc().get_RGBDescriptor();
       const auto& data = sdb.data();
@@ -976,16 +1103,42 @@ bool TexUnpackSurface::TexOrSubImage(bool isSubImage, bool needsRespec,
         return false;
       }
       surf = texture->GetAsSurface();
-    } else {
-      MOZ_ASSERT_UNREACHABLE("Unexpected surface descriptor!");
+    } else if (SDIsExternalImage(sd)) {
+      const auto& sdei = sd.get_SurfaceDescriptorExternalImage();
+      if (auto* sharedSurfacesHolder = webgl->GetSharedSurfacesHolder()) {
+        surf = sharedSurfacesHolder->Get(sdei.id());
+      }
+      if (!surf) {
+        // Most likely the content process crashed before it was able to finish
+        // sharing the surface with the compositor process.
+        gfxCriticalNote << "TexUnpackSurface failed to get ExternalImage";
+        return false;
+      }
+    } else if (AllowBlitSd(webgl, mDesc.imageTarget, level,
+                           {xOffset, yOffset, zOffset}, dui->internalFormat,
+                           dstPI, false, true, true, true) &&
+               BlitSd(sd, isSubImage, needsRespec, tex, level, dui, xOffset,
+                      yOffset, zOffset, dstPI, out_error)) {
+      // The SD wasn't an RGBA shmem, but were able to blit the SD directly to
+      // the texture.
+      return true;
+    } else if (mDesc.sourceSurf) {
+      // In case neither a wrapper was created nor a blit succeeded, check for a
+      // backup source surface.
+      surf = mDesc.sourceSurf->GetDataSurface();
     }
     if (!surf) {
       gfxCriticalError() << "TexUnpackSurface failed to create wrapping "
                             "DataSourceSurface for Shmem.";
       return false;
     }
-  } else {
-    surf = mDesc.dataSurf;
+  } else if (mDesc.sourceSurf) {
+    surf = mDesc.sourceSurf->GetDataSurface();
+    if (!surf) {
+      gfxCriticalError() << "TexUnpackSurface failed to get data for "
+                            "sourceSurf.";
+      return false;
+    }
   }
 
   ////
